@@ -19,9 +19,9 @@
 #'  submitted to Computational Statistics and Data Analysis.
 #' @export
 #' @examples
+#' data("simu_warp")
 #' data("simu_data")
-#' out = time_warping(simu_data$f,simu_data$time)
-#' vfpca = vertFPCA(out$fn,simu_data$time,out$qn,no = 3)
+#' vfpca = vertFPCA(simu_warp$fn,simu_data$time,simu_warp$qn,no = 3)
 vertFPCA <- function(fn,time,qn,no,showplot = TRUE){
 	# Parameters
 	coef = -2:2
@@ -32,7 +32,7 @@ vertFPCA <- function(fn,time,qn,no,showplot = TRUE){
 	mq_new = rowMeans(qn)
 	m_new = sign(fn[round(length(time)/2),])*sqrt(abs(fn[round(length(time)/2),]))  # scaled version
 	mqn = c(mq_new,mean(m_new))
-	K = cov_samp(t(rbind(qn,m_new))) #out$sigma
+	K = cov(t(rbind(qn,m_new))) #out$sigma
 	
 	out = svd(K)
 	s = out$d
